/**
 * \file shape.h
 * \brief class  TO manage shape generation
 * \author Luis Alvarez \n \n
*/


#ifndef SHAPE_GENERATION_H
#define SHAPE_GENERATION_H
#include "color.h"
#include "shape.h"
#include <iostream>
#include <vector>
#include <math.h>

#define ami_PI 3.14159265358979323846


// GENERATION OF A KANDISKY CIRCLE
shape kandisky_circle_generator(float radius,int Nlevels,int CircleNpoints);
// GENERATION OF A KANDISKY RECTANGLE
shape kandisky_rectangle_generator(float width,float height,int Nlevels,float min_distance);
// GENERATION OF A KANDISKY CIRCLE FRAME
shape kandisky_circle_frame_generator(point2d  pmin, point2d pmax,int NCircles_x, int NCircles_y, int Nlevels,int CircleNpoints);
// GENERATION OF A KANDISKY RECTANGLE FRAME
shape kandisky_rectangle_frame_generator(point2d  pmin, point2d pmax,int NCircles_x, int NCircles_y, int Nlevels,float min_distance);


class leaf{
  public :
  triangulation T0; /// triangulation where the new triangulation has to be included (if T0 has not triangles, the test is not performed)
//  triangulation T; /// new triangulation created using parameters
  point2d p0; /// point to start iterations
  float dt; /// distance between consecutive points
  float a0; /// initial angle orientation of the curve orthogonal vector
  float a1; /// final angle orientation of the curve orthogonal vector
  float dal;float dar; /// interval of variation of "a" where the variation is randomly selected
  float r0; /// initial radius of the curve cross-section
  float r1; /// final radius of the curve cross-section
  float drl;float drr; /// interval of variation of "r" where the variation is randomly selected
  float min_r; /// min r value
  int Niter;
  leaf(){p0.x=p0.y=0.; dt=0.1; a0=a1=0.; dal=-0.5; dar=0.5; r0=r1=0.1; drl=-0.02; drr=0.02; min_r=0.01; Niter=25;}
  leaf(int type); /// constructor using an integer type
  leaf(int type,triangulation &T1,int index_triangle,int index_segment); /// constructor using a tangent segment to a triangulation
  triangulation T(); /// build the triangulation using the parameters
  shape new_shape(){return shape(T());}
};

class kandisky_circle{
  public :
  float radius;
  int Nlevels;
  int CircleNpoints;
  kandisky_circle(){radius=0.05; Nlevels=3; CircleNpoints=20;}
  kandisky_circle(int type){radius=0.05; Nlevels=3; CircleNpoints=20;};
  shape new_shape(){return kandisky_circle_generator(radius,Nlevels,CircleNpoints);}
};

class kandisky_rectangle{
  public :
  float width;
  float height;
  int Nlevels;
  float min_distance;
  kandisky_rectangle(){width=height=0.1; Nlevels=3;min_distance=0.01;}
  kandisky_rectangle(int type){width=height=0.1; Nlevels=3;min_distance=0.01;};
  shape new_shape(){return kandisky_rectangle_generator(width,height,Nlevels,min_distance);}
};

/**
 * \class  shape_generator
 * \brief class  declare procedures to define new shapes
 * \author Luis Alvarez
 */
class shape_generator{
  public :
  int shape_type;
  triangulation T0; /// triangulation where the new triangulation has to be included (if T0 has not triangles, the test is not performed)

  /// KANDISKY CIRCLE PARAMETERS
  kandisky_circle Kc;  /// shape_type from 100 to 199

  /// KANDISKY RECTANGLE PARAMETERS
  kandisky_rectangle Kr; /// shape_type from 200 to 299

  /// LEAF PARAMETERS
  leaf Leaf; /// shape_type from 300 to 399

  shape new_shape(int shape_type_new);
  shape new_shape(int shape_type_new,triangulation &T1,int index_triangle,int index_segment,float min_distance=0.01,float zoom=1.);
  shape new_shape(int &shape_database,int shape_index);

};



/// CONTOURING A SHAPE WITH ANOTHER SHAPE TYPE
void contouring(shape &S0,shape &S,float min_distance,int shape_type,int Ntries,float distance=0.01,float zoom=1.);

/// READ A COLLECTION OF SHAPES FROM TRIANGULIATION STORED IN A REPOSITORY
void ReadShapeDataBase(vector <shape> &SDB,char RepositoryName[]);
/// ADD TO A COLLECTION OF PALETTES GIVEN BY BMP IMAGES STORED IN A REPOSITORY
void ReadPaletteDataBase(vector <imageBMP> &PDB,char RepositoryName[]);


#endif
